@@ -14,12 +14,12 @@ test_that("allDifferences_asreml4", {
                    random=~Blocks/Wplots,
                    data=Oats.dat)
   testthat::expect_equal(length(m1.asr$vparameters),3)
-  current.asrt <- asrtests(m1.asr)
+  current.asrt <- as.asrtests(m1.asr)
+  wald.tab <-  current.asrt$wald.tab
+  den.df <- wald.tab[match("Variety", rownames(wald.tab)), "denDF"]
   
   #Test for as.alldiffs
   Var.pred <- predict(m1.asr, classify="Nitrogen:Variety", sed=TRUE)
-  wald.tab <-  current.asrt$wald.tab
-  den.df <- wald.tab[match("Variety", rownames(wald.tab)), "denDF"]
   Var.diffs <- as.alldiffs(predictions = Var.pred$pvals, 
                            sed = Var.pred$sed, 
                            classify = "Nitrogen:Variety", response = "Yield", tdf = den.df)
@@ -35,6 +35,8 @@ test_that("allDifferences_asreml4", {
                                    sortFactor = "Variety", decreasing = TRUE)
   testthat::expect_true(is.alldiffs(Var.sort.diffs))
   testthat::expect_true(validAlldiffs(Var.sort.diffs))
+  testthat::expect_equal(length(attr(Var.sort.diffs$predictions, which = "heading")),4)
+  testthat::expect_true("asreml.predict" %in% class(Var.sort.diffs$predictions))
   testthat::expect_equal(length(attr(Var.sort.diffs, which = "sortOrder")),3)
   testthat::expect_true(as.character(Var.sort.diffs$predictions$Variety[1]) == "Marvellous" & 
                           as.character(Var.sort.diffs$predictions$Variety[2]) == "Golden Rain")
@@ -50,6 +52,8 @@ test_that("allDifferences_asreml4", {
   Var.reord.diffs <- reorderClassify(Var.diffs, newclassify = "Variety:Nitrogen")
   testthat::expect_true(as.character(Var.reord.diffs$predictions$Variety[1]) == "Victory" &
                           as.character(Var.reord.diffs$predictions$Variety[2]) == "Victory")
+  testthat::expect_equal(length(attr(Var.reord.diffs$predictions, which = "heading")),4)
+  testthat::expect_true("asreml.predict" %in% class(Var.reord.diffs$predictions))
   
   #Test for re-order factors and sort
   Var.both.diffs <- allDifferences(predictions = Var.pred$pvals,
@@ -66,7 +70,25 @@ test_that("allDifferences_asreml4", {
   #Test a single factor prediction
   diffsN <- predictPlus(m1.asr, classify = "Nitrogen", tables = "none")
   testthat::expect_true(validAlldiffs(diffsN))
-  })
+  
+  #Test single factor linear.transform
+  Var.pred <- predict(m1.asr, classify="Nitrogen:Variety", vcov=TRUE)
+  Var.diffs <- allDifferences(predictions = Var.pred$pvals,
+                              classify = "Nitrogen:Variety", 
+                              vcov = Var.pred$vcov, tdf = den.df)
+  Var.diffs.one <- linTransform(Var.diffs, linear.transformation = ~Nitrogen,
+                                error.intervals = "half", tables = "none")
+  testthat::expect_true(all(abs(Var.diffs.one$LSD - 9.883479) < 1e-06))
+  testthat::expect_true(all(abs(Var.diffs.one$LSD - 
+                              attr(Var.diffs.one$predictions, which = "meanLSD")) < 1E-06))
+  #Test LSDby not in linear.transformation
+  testthat::expect_warning(Var.diffs.by <- linTransform(Var.diffs, 
+                                                        linear.transformation = ~Nitrogen,
+                                                        error.intervals = "half", 
+                                                        meanLSD.type = "factor", 
+                                                        LSDby = "Variety", 
+                                                        tables = "none"))
+})
 
 cat("#### Test for sort.alldiffs on Smarthouse with asreml4\n")
 test_that("sort.alldiffs4", {
@@ -82,7 +104,7 @@ test_that("sort.alldiffs4", {
                    random=~Replicate/Mainplot/Subplot,
                    data=Smarthouse.dat)
   testthat::expect_equal(length(m1.asr$vparameters),4)
-  current.asrt <- asrtests(m1.asr)
+  current.asrt <- as.asrtests(m1.asr)
   current.asrt <- rmboundary(current.asrt)
   m <- current.asrt$asreml.obj
   testthat::expect_equal(length(m$vparameters),3)
@@ -127,7 +149,7 @@ test_that("sort.alldiffs4", {
   testthat::expect_equal(nrow(diffs.sort$predictions),120)
   testthat::expect_equal(ncol(diffs.sort$predictions),8)
   testthat::expect_equal(as.character(diffs.sort$predictions$Genotype[1]),"Gladius")
-  testthat::expect_equal(length(attributes(diffs.sort)),9)
+  testthat::expect_equal(length(attributes(diffs.sort)),10)
   testthat::expect_equal(length(attr(diffs.sort, which = "sortOrder")),10)
   
   #Test sort.alldiffs with supplied sortOrder
@@ -135,7 +157,7 @@ test_that("sort.alldiffs4", {
                    random=~Replicate/Mainplot/Subplot,
                    data=Smarthouse.dat)
   testthat::expect_equal(length(m1.asr$vparameters),4)
-  current.asrt <- asrtests(m2.asr)
+  current.asrt <- as.asrtests(m2.asr)
   diffs2.sort <- predictPlus(m2.asr, classify = "Genotype:A:B", 
                              wald.tab = current.asrt$wald.tab,
                              error.intervals = "Stand", tables = "none",
@@ -176,9 +198,9 @@ test_that("sort.alldiffsWater4", {
   m1.asr <- asreml(fixed = pH ~ Benches + (Sources * (Type + Species)), 
                         random = ~ Benches:MainPlots,
                         keep.order=TRUE, data= WaterRunoff.dat)
-  current.asrt <- asrtests(m1.asr, NULL, NULL)
+  current.asrt <- as.asrtests(m1.asr, NULL, NULL)
   testthat::expect_equal(length(m1.asr$vparameters),2)
-  current.asrt <- asrtests(m1.asr)
+  current.asrt <- as.asrtests(m1.asr)
   current.asrt <- rmboundary(current.asrt)
   m1.asr <- current.asrt$asreml.obj
   testthat::expect_equal(length(m1.asr$vparameters),1)
@@ -208,7 +230,7 @@ test_that("sort.alldiffsWater4", {
   testthat::expect_equal(nrow(TS.diffs.sort$predictions),20)
   testthat::expect_equal(ncol(TS.diffs.sort$predictions),7)
   testthat::expect_equal(as.character(TS.diffs.sort$predictions$Sources[1]),"Recycled water")
-  testthat::expect_equal(length(attributes(TS.diffs.sort)),9)
+  testthat::expect_equal(length(attributes(TS.diffs.sort)),10)
   testthat::expect_equal(length(attr(TS.diffs.sort, which = "sortOrder")),6)
   
   #Test sort.alldiffs with supplied sortOrder
@@ -216,7 +238,7 @@ test_that("sort.alldiffsWater4", {
                    random = ~ Benches:MainPlots,
                    keep.order=TRUE, data= WaterRunoff.dat)
   testthat::expect_equal(length(m2.asr$vparameters),2)
-  current.asrt <- asrtests(m2.asr)
+  current.asrt <- as.asrtests(m2.asr)
   diffs2.sort <- predictPlus(m2.asr, classify = "Sources:Type", 
                              pairwise = FALSE, error.intervals = "Stand", 
                              tables = "none", present = c("Type","Species","Sources"),
@@ -241,7 +263,7 @@ test_that("sort.alldiffs4", {
                    random=~Blocks/Wplots,
                    data=Oats.dat)
   testthat::expect_equal(length(m1.asr$vparameters),3)
-  current.asrt <- asrtests(m1.asr)
+  current.asrt <- as.asrtests(m1.asr)
   
   #Test for as.alldiffs
   Var.pred <- predict(m1.asr, classify="Nitrogen:Variety", sed=TRUE)
@@ -263,7 +285,7 @@ test_that("sort.alldiffs4", {
   testthat::expect_equal(nrow(diffs$predictions),12)
   testthat::expect_equal(ncol(diffs$predictions),7)
   testthat::expect_equal(as.character(diffs$predictions$Variety[1]),"Victory")
-  testthat::expect_equal(length(attributes(diffs)),7)
+  testthat::expect_equal(length(attributes(diffs)),8)
   testthat::expect_true(is.null(attr(diffs, which = "sortOrder")))
   
   testthat::expect_silent(plotPredictions(data = diffs$predictions, 
@@ -287,7 +309,7 @@ test_that("sort.alldiffs4", {
                    data=Oats.dat)
   
   testthat::expect_equal(length(mx.asr$vparameters),3)
-  current.asrt <- asrtests(mx.asr)
+  current.asrt <- as.asrtests(mx.asr)
   print(current.asrt)
   
   diffs <- predictPresent(mx.asr, terms = "xNitrogen:Variety", 
@@ -301,7 +323,7 @@ test_that("sort.alldiffs4", {
   testthat::expect_equal(nrow(diffs$xNitrogen.Variety$predictions),12)
   testthat::expect_equal(ncol(diffs[[1]]$predictions),7)
   testthat::expect_equal(as.character(diffs[[1]]$predictions$Variety[[1]]),"Marvellous")
-  testthat::expect_equal(length(attributes(diffs$xNitrogen.Variety)),9)
+  testthat::expect_equal(length(attributes(diffs$xNitrogen.Variety)),10)
   testthat::expect_equal(length(attr(diffs[[1]], which = "sortOrder")),3)
   
   #Test for predictPlus with sortFactor
@@ -316,7 +338,7 @@ test_that("sort.alldiffs4", {
   testthat::expect_equal(nrow(diffs$predictions),12)
   testthat::expect_equal(ncol(diffs$predictions),7)
   testthat::expect_equal(as.character(diffs$predictions$Variety[1]),"Marvellous")
-  testthat::expect_equal(length(attributes(diffs)),9)
+  testthat::expect_equal(length(attributes(diffs)),10)
   testthat::expect_true(all(attr(diffs, which = "sortOrder") == 
                               levels(diffs$predictions$Variety)))
   testthat::expect_true(all(attr(diffs, which = "sortOrder") == 
@@ -346,7 +368,7 @@ test_that("subset.alldiffs4", {
                    random=~Replicate/Mainplot/Subplot,
                    data=Smarthouse.dat)
   testthat::expect_equal(length(m1.asr$vparameters),4)
-  current.asrt <- asrtests(m1.asr)
+  current.asrt <- as.asrtests(m1.asr)
   current.asrt <- rmboundary(current.asrt)
   m <- current.asrt$asreml.obj
   testthat::expect_equal(length(m$vparameters),3)
@@ -359,7 +381,7 @@ test_that("subset.alldiffs4", {
   testthat::expect_equal(nrow(diffs$predictions),120)
   testthat::expect_equal(ncol(diffs$predictions),8)
   testthat::expect_equal(as.character(diffs$predictions$Genotype[1]),"Axe")
-  testthat::expect_equal(length(attributes(diffs)),7)
+  testthat::expect_equal(length(attributes(diffs)),8)
   testthat::expect_true(is.null(attr(diffs, which = "sortOrder")))
   
   #Form subset
@@ -372,7 +394,7 @@ test_that("subset.alldiffs4", {
   testthat::expect_equal(ncol(diffs.subs$predictions),8)
   testthat::expect_false(any(diffs.subs$predictions$Genotype %in% c("Excalibur","Espada")))
   testthat::expect_false(any(diffs.subs$predictions$B %in% c("D3","D4")))
-  testthat::expect_equal(length(attributes(diffs.subs)),7)
+  testthat::expect_equal(length(attributes(diffs.subs)),8)
   
   #Test subset with removal of vars
   diffs.subs <- subset(diffs, subset = A == "N1" & B == "D2", rmClassifyVars = c("A","B"))
@@ -388,7 +410,7 @@ test_that("subset.alldiffs4", {
   current.asr <- asreml(fixed = pH ~ Benches + (Sources * (Type + Species)), 
                         random = ~ Benches:MainPlots,
                         data= WaterRunoff.dat)
-  current.asrt <- asrtests(current.asr, NULL, NULL)
+  current.asrt <- as.asrtests(current.asr, NULL, NULL)
   diffs <- predictPlus.asreml(classify = "Sources:Type", 
                               asreml.obj = current.asr, tables = "none", 
                               wald.tab = current.asrt$wald.tab, 
@@ -405,7 +427,7 @@ test_that("subset.alldiffs4", {
   testthat::expect_equal(ncol(diffs.subs$predictions),7)
   testthat::expect_false(any(diffs.subs$predictions$Sources == "Tap water"))
   testthat::expect_false(any(diffs.subs$predictions$B %in% c("Landscape","Culinary")))
-  testthat::expect_equal(length(attributes(diffs.subs)),7)
+  testthat::expect_equal(length(attributes(diffs.subs)),8)
 })
 
 cat("#### Test for facCombine.alldiffs on Ladybird with asreml4\n")
@@ -423,7 +445,7 @@ test_that("facCombine.alldiffs4", {
                    data = Ladybird.dat)
 
   testthat::expect_equal(length(m1.asr$vparameters),2)
-  current.asrt <- asrtests(m1.asr)
+  current.asrt <- as.asrtests(m1.asr)
   testthat::expect_true(validAsrtests(current.asrt))
 
   HCL.pred <- asreml::predict.asreml(m1.asr, classify="Host:Cadavers:Ladybird", 
@@ -440,8 +462,8 @@ test_that("facCombine.alldiffs4", {
 
 
 
-cat("#### Test for linear.transformation with asreml4\n")
-test_that("linear.transformation_asreml4", {
+cat("#### Test for linear.transformation on Oats with asreml4\n")
+test_that("linear.transform_Oats_asreml4", {
   skip_if_not_installed("asreml")
   skip_on_cran()
   library(asreml)
@@ -453,13 +475,15 @@ test_that("linear.transformation_asreml4", {
                    random=~Blocks/Wplots,
                    data=Oats.dat)
   testthat::expect_equal(length(m1.asr$vparameters),3)
-  current.asrt <- asrtests(m1.asr)
+  current.asrt <- as.asrtests(m1.asr)
   
   #Test store of vcov by predictPlus
   diffs <- predictPlus(m1.asr, classify = "Nitrogen:Variety", Vmatrix = TRUE, 
                        wald.tab = current.asrt$wald.tab,
                        error.intervals = "Stand", tables = "none")
   testthat::expect_is(diffs, "alldiffs")
+  testthat::expect_equal(length(attr(diffs$predictions, which = "heading")),4)
+  testthat::expect_true("asreml.predict" %in% class(diffs$predictions))
   testthat::expect_equal(nrow(diffs$vcov),12)
   testthat::expect_true(all(colnames(diffs$vcov)[1:2] %in% c("0,Victory", "0,Golden Rain")))
   testthat::expect_true(all(abs((diffs$vcov[1,1:2] - c(82.93704, 35.74618))) < 1e-4))
@@ -474,6 +498,8 @@ test_that("linear.transformation_asreml4", {
                          wald.tab = current.asrt$wald.tab,
                          error.intervals = "Conf", tables = "none")
   testthat::expect_is(diffs.L, "alldiffs")
+  testthat::expect_equal(length(attr(diffs.L$predictions, which = "heading")),5)
+  testthat::expect_true("asreml.predict" %in% class(diffs.L$predictions))
   testthat::expect_true(abs((diffs$vcov[1,1] - diffs$vcov[4,1]) + 
                               (diffs$vcov[4,4] - diffs$vcov[1,4]) - diffs.L$vcov[1,1]) < 1e-04)
   testthat::expect_equal(as.character(diffs.L$predictions$Combination[1]), "0.2,Victory")
@@ -491,6 +517,8 @@ test_that("linear.transformation_asreml4", {
                            LSDby = "Nitrogen",
                            tables = "none")
   testthat::expect_is(diffs.mod, "alldiffs")
+  testthat::expect_equal(length(attr(diffs.mod$predictions, which = "heading")),5)
+  testthat::expect_true("asreml.predict" %in% class(diffs.mod$predictions))
   testthat::expect_true(is.null(diffs.mod$vcov))
   
   m2.asr <- asreml(Yield ~ Nitrogen+Variety, 
@@ -501,7 +529,15 @@ test_that("linear.transformation_asreml4", {
                                   preds$predicted.value) < 1e-04))
   testthat::expect_true(abs(diffs.mod$predictions$standard.error[1] - 
                               preds$std.error[1]) > 0.01)
+})
   
+cat("#### Test for linear.transformation on WaterRunoff with asreml4\n")
+test_that("linear.transform_WaterRunoff_asreml4", {
+  skip_if_not_installed("asreml")
+  skip_on_cran()
+  library(asreml)
+  library(asremlPlus)
+  library(dae)
   #Test example in manual
   data(WaterRunoff.dat)
   #Run analysis and produce alldiffs object
@@ -509,7 +545,7 @@ test_that("linear.transformation_asreml4", {
   current.asr <- asreml(fixed = pH ~ Benches + (Sources * (Type + Species)), 
                         random = ~ Benches:MainPlots,
                         data= WaterRunoff.dat)
-  current.asrt <- asrtests(current.asr, NULL, NULL)
+  current.asrt <- as.asrtests(current.asr, NULL, NULL)
   diffs <- predictPlus(classify = "Sources:Species", Vmatrix = TRUE, 
                        asreml.obj = current.asr, tables = "none", 
                        wald.tab = current.asrt$wald.tab, 
@@ -536,6 +572,10 @@ test_that("linear.transformation_asreml4", {
                                                    classify = "Sources:Species",
                                                    linear.transformation = L,
                                                    tables = "none"))
+  #check for zero seds and their removal
+  ksed <- na.omit(as.vector(diffs.L$sed))
+  testthat::expect_true(length(ksed[ksed/max(ksed, na.rm = TRUE) <= 1e-6]) == 66)
+  testthat::expect_true(abs(diffs.L$LSD["minLSD"] - 0.1246359) < 1e-06)
   testthat::expect_true(all(abs(diffs.L$predictions$predicted.value[c(1,6,11,16,21,28)] - 
                                   (diffs.sub$predictions$predicted.value[1] - diffs.sub$predictions$predicted.value[6])) < 1e-06))
   
@@ -545,7 +585,7 @@ test_that("linear.transformation_asreml4", {
   current.asr <- asreml(fixed = pH ~ Benches + (Sources * (Type + Species)), 
                         random = ~ Benches:MainPlots,
                         data= WaterRunoff.dat)
-  current.asrt <- asrtests(current.asr, NULL, NULL)
+  current.asrt <- as.asrtests(current.asr, NULL, NULL)
   #Get additive predictions directly using predictPlus
   diffs.sub <- predictPlus.asreml(classify = "Sources:Species", Vmatrix = TRUE, 
                                   linear.transformation = ~ Sources + Species,
@@ -628,7 +668,7 @@ test_that("linear.transformation_asreml4", {
                                      na.action=na.method(y="include", x="include"),
                                      maxiter=50))
     summary(HEB25.asr)$varcomp
-    current.asrt <- asrtests(HEB25.asr)
+    current.asrt <- as.asrtests(HEB25.asr)
     current.asrt <- rmboundary(current.asrt)
     current.asr <- current.asrt$asreml.obj
     wald.tab <- recalcWaldTab(asrtests.obj=current.asrt, dDF.na="residual")
